/*
 * LIBPKI - OpenSource PKI library
 * by Massimiliano Pala (madwolf@openca.org) and OpenCA project
 *
 * Copyright (c) 2001-2007 The OpenCA Project.  All rights reserved.
 *
 * ====================================================================
 *
 * This product includes cryptographic software written by Eric Young
 * (eay@cryptsoft.com).  This product includes software written by Tim
 * Hudson (tjh@cryptsoft.com).
 *
 */

#ifndef __OPENCA_SOCK_WRAP_H
#define __OPENCA_SOCK_WRAP_H

#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>

#include <netinet/in.h>
#include <arpa/inet.h>

#include <errno.h>
#include <strings.h>
#include <string.h>
#include <stdio.h>

#include <unistd.h>
#include <syslog.h>

#include <libpki/net/url.h>

#define INTERRUPTED_BY_SIGNAL (errno == EINTR || errno == ECHILD)

#define SA struct sockaddr

/* Public Functions */
int PKI_NET_socket ( int family, int type, int protocol );
int PKI_NET_listen ( char *host, int port );
int PKI_NET_accept ( int sock, int timeout );
int PKI_NET_open ( URL *url, int timeout );
int PKI_NET_close ( int sock );
ssize_t PKI_NET_write (int fd, void *bufptr, size_t nbytes);
ssize_t PKI_NET_read (int fd, void *bufptr, size_t nbytes, int timeout);
PKI_MEM *PKI_NET_get_data ( int fd, int timeout, size_t max_size );

/* Internal Socket Wrapping functions */
int _Listen (char *hostname, int port);
int _Accept (int listen_sockfd, SA *cliaddr, socklen_t *addrlenp);
ssize_t _Read (int fd, void *bufptr, size_t nbytes);
ssize_t _Write (int fd, void *bufptr, size_t nbytes);
int _Select (int maxfdp1, fd_set *readset, fd_set *writeset, 
			fd_set *exceptset, struct timeval *timeout);

/* Externally available functions */
int inet_connect ( URL *url );
int inet_close ( int fd );

#endif
